@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2014 Gentoo Foundation
+ * Copyright 2005-2018 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2008 Ned Ludd        - <solar@gentoo.org>
@@ -189,15 +189,21 @@ usage(int status, const char *flags, struct option const opts[],
 static void
 version_barf(void)
 {
+	const char *eprefixid = "";
+
 #ifndef VERSION
 # define VERSION "git"
 #endif
 #ifndef VCSID
 # define VCSID "<unknown>"
 #endif
+
+	if (strlen(CONFIG_EPREFIX) > 1)
+		eprefixid = "configured for " CONFIG_EPREFIX "\n";
 	printf("portage-utils-%s: %s\n"
+	       "%s"
 	       "%s written for Gentoo by <solar and vapier @ gentoo.org>\n",
-	       VERSION, VCSID, argv0);
+	       VERSION, VCSID, eprefixid, argv0);
 	exit(EXIT_SUCCESS);
 }
 
@@ -510,7 +516,11 @@ read_repos_conf(const char *configroot, const char *repos_conf)
 		for (i = 0; i < count; ++i) {
 			const char *name = confs[i]->d_name;
 
-			if (name[0] == '.')
+			if (name[0] == '.' || name[0] == '\0')
+				continue;
+
+			/* Exclude backup files (aka files with ~ as postfix). */
+			if (name[strlen(name) - 1] == '~')
 				continue;
 
 #ifdef DT_UNKNOWN
@@ -629,16 +639,19 @@ set_portage_env_var(env_vars *var, const char *value)
 	}
 }
 
-/* Helper to read a portage env file (e.g. make.conf) */
+/* Helper to read a portage env file (e.g. make.conf), or recursively if
+ * it points to a directory */
 static void
 read_portage_env_file(const char *configroot, const char *file, env_vars vars[])
 {
 	size_t i, buflen, line, configroot_len, file_len;
 	FILE *fp;
+	struct dirent **dents;
+	int dentslen;
 	char *buf, *s, *p;
 
 	if (getenv("DEBUG"))
-		fprintf(stderr, "profile %s\n", file);
+		fprintf(stderr, "profile %s/%s\n", configroot, file);
 
 	configroot_len = strlen(configroot);
 	file_len = strlen(file);
@@ -649,6 +662,24 @@ read_portage_env_file(const char *configroot, const char *file, env_vars vars[])
 	memcpy(buf + configroot_len, file, file_len);
 	buf[buflen - 1] = '\0';
 
+	if ((dentslen = scandir(buf, &dents, NULL, alphasort)) > 0) {
+		int di;
+		struct dirent *d;
+		char npath[_Q_PATH_MAX];
+
+		/* recurse through all files */
+		for (di = 0; di < dentslen; di++) {
+			d = dents[di];
+			if (d->d_name[0] == '.' || d->d_name[0] == '\0' ||
+					d->d_name[strlen(d->d_name) - 1] == '~')
+				continue;
+			snprintf(npath, sizeof(npath), "%s/%s", file, d->d_name);
+			read_portage_env_file(configroot, npath, vars);
+		}
+		scandir_free(dents, dentslen);
+		goto done;
+	}
+
 	fp = fopen(buf, "r");
 	if (fp == NULL)
 		goto done;
@@ -933,6 +964,21 @@ initialize_portage_env(void)
 		}
 	}
 
+	/* Make sure ROOT always ends in a slash */
+	var = &vars_to_read[0];
+	if ((*var->value.s)[var->value_len - 1] != '/') {
+		portroot = xrealloc(portroot, var->value_len + 2);
+		portroot[var->value_len] = '/';
+		portroot[var->value_len + 1] = '\0';
+	}
+
+	char *orig_main_overlay = main_overlay;
+	read_repos_conf(configroot, "/etc/portage/repos.conf");
+	if (orig_main_overlay != main_overlay)
+		free(orig_main_overlay);
+	if (array_cnt(overlays) == 0)
+		xarraypush_str(overlays, main_overlay);
+
 	if (getenv("DEBUG")) {
 		for (i = 0; vars_to_read[i].name; ++i) {
 			var = &vars_to_read[i];
@@ -945,21 +991,6 @@ initialize_portage_env(void)
 		}
 	}
 
-	/* Make sure ROOT always ends in a slash */
-	var = &vars_to_read[0];
-	if ((*var->value.s)[var->value_len - 1] != '/') {
-		portroot = xrealloc(portroot, var->value_len + 2);
-		portroot[var->value_len] = '/';
-		portroot[var->value_len + 1] = '\0';
-	}
-
-	char *orig_main_overlay = main_overlay;
-	read_repos_conf(configroot, CONFIG_EPREFIX "etc/portage/repos.conf");
-	if (orig_main_overlay != main_overlay)
-		free(orig_main_overlay);
-	if (array_cnt(overlays) == 0)
-		xarraypush_str(overlays, main_overlay);
-
 	if (getenv("PORTAGE_QUIET") != NULL)
 		setup_quiet();
 
@@ -1416,9 +1447,11 @@ get_vdb_atoms(int fullcpv)
 		if ((dfd = scandirat(ctx->vdb_fd, cat[j]->d_name, &pf, q_vdb_filter_pkg, alphasort)) < 0)
 			continue;
 		for (i = 0; i < dfd; i++) {
-			int blen = snprintf(buf, sizeof(buf), "%s/%s/SLOT", cat[j]->d_name, pf[i]->d_name);
-			if (blen >= sizeof(buf)) {
-				warnf("unable to parse long package: %s/%s", cat[j]->d_name, pf[i]->d_name);
+			int blen = snprintf(buf, sizeof(buf), "%s/%s/SLOT",
+					cat[j]->d_name, pf[i]->d_name);
+			if (blen < 0 || (size_t)blen >= sizeof(buf)) {
+				warnf("unable to parse long package: %s/%s",
+						cat[j]->d_name, pf[i]->d_name);
 				continue;
 			}
 
