@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2014 Gentoo Foundation
+ * Copyright 2005-2018 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
@@ -354,8 +354,8 @@ int qcheck_main(int argc, char **argv)
 		.undo_prelink = false,
 	};
 
-	while ((i = GETOPT_LONG(QCHECK, qcheck, "")) != -1) {
-		switch (i) {
+	while ((ret = GETOPT_LONG(QCHECK, qcheck, "")) != -1) {
+		switch (ret) {
 		COMMON_GETOPTS_CASES(qcheck)
 		case 's': {
 			regex_t regex;
@@ -375,7 +375,7 @@ int qcheck_main(int argc, char **argv)
 
 	argc -= optind;
 	argv += optind;
-	for (i = 0; i < argc; ++i) {
+	for (i = 0; i < (size_t)argc; ++i) {
 		atom = atom_explode(argv[i]);
 		if (!atom)
 			warn("invalid atom: %s", argv[i]);
