@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2014 Gentoo Foundation
+ * Copyright 2005-2018 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
@@ -89,6 +89,7 @@ show_merge_times(char *package, const char *logfile, int average, char human_rea
 	count = merge_time = 0;
 	cat[0] = 0;
 
+	/* setup cat and pkg vars */
 	if ((p = strchr(package, '/')) != NULL) {
 		pkg = p + 1;
 		strncpy(cat, package, sizeof(cat));
@@ -103,51 +104,81 @@ show_merge_times(char *package, const char *logfile, int average, char human_rea
 		return 1;
 	}
 
+	/* loop over lines searching for cat/pkg */
+	parallel_emerge = 0;
 	while (fgets(buf[0], sizeof(buf[0]), fp) != NULL) {
-		if (strstr(buf[0], pkg) == NULL)
-			continue;
-
 		if ((p = strchr(buf[0], '\n')) != NULL)
-			*p = 0;
+			*p = '\0';
 		if ((p = strchr(buf[0], ':')) == NULL)
 			continue;
-		*p = 0;
+		*p++ = '\0';
+		if (strstr(p, pkg) == NULL)
+			continue;
+
 		t[0] = atol(buf[0]);
 		if (t[0] < start_time || t[0] > end_time)
 			continue;
-		strcpy(buf[1], p + 1);
+
+		/* copy message (stripping timestamp) */
+		strncpy(buf[1], p, BUFSIZ);
 		rmspace(buf[1]);
+
+		if (strncmp(buf[1], "Started emerge on:", 18) == 0) {
+			/* a parallel emerge was launched */
+			parallel_emerge++;
+			continue;
+		}
+
+		if (strncmp(buf[1], "*** terminating.", 16) == 0) {
+			if (parallel_emerge > 0) {
+				/* a parallel emerge has finished */
+				parallel_emerge--;
+				continue;
+			} else {
+				/* the main emerge was stopped? if there's more lines
+				 * this file is just corrupt or truncated at the front */
+				continue;
+			}
+		}
+
 		if (strncmp(buf[1], ">>> emerge (", 12) == 0) {
+			/* construct the matching end marker */
 			snprintf(ep, BUFSIZ, "completed %s", &buf[1][4]);
 
-			char matched = 0;
-			if ((p = strchr(buf[1], ')')) == NULL)
+			/* skip over "(X of Y)" */
+			if ((p = strchr(buf[1], ')')) == NULL) {
+				*ep = '\0';
 				continue;
-			*p = 0;
-			strcpy(buf[0], p + 1);
+			}
+			*p++ = '\0';
+
+			/* get the package as atom */
+			strncpy(buf[0], p, BUFSIZ);
 			rmspace(buf[0]);
-			if ((p = strchr(buf[0], ' ')) == NULL)
+			if ((p = strchr(buf[0], ' ')) == NULL) {
+				*ep = '\0';
 				continue;
-			*p = 0;
-			if ((atom = atom_explode(buf[0])) == NULL)
+			}
+			*p = '\0';
+			if ((atom = atom_explode(buf[0])) == NULL) {
+				*ep = '\0';
 				continue;
+			}
 
-			if (*cat) {
-				if ((strcmp(cat, atom->CATEGORY) == 0) && (strcmp(pkg, atom->PN) == 0))
-					matched = 1;
-			} else if (strcmp(pkg, atom->PN) == 0)
-				matched = 1;
-
-			if (matched) {
-				parallel_emerge = 0;
+			/* match atom against our search */
+			if ((*cat && ((strcmp(cat, atom->CATEGORY) == 0) &&
+							(strcmp(pkg, atom->PN) == 0))) ||
+					(strcmp(pkg, atom->PN) == 0))
+			{
 				while (fgets(buf[0], sizeof(buf[0]), fp) != NULL) {
 					if ((p = strchr(buf[0], '\n')) != NULL)
-						*p = 0;
+						*p = '\0';
 					if ((p = strchr(buf[0], ':')) == NULL)
 						continue;
-					*p = 0;
+					*p++ = '\0';
+
 					t[1] = atol(buf[0]);
-					strcpy(buf[1], p + 1);
+					strcpy(buf[1], p);
 					rmspace(buf[1]);
 
 					if (strncmp(buf[1], "Started emerge on:", 18) == 0) {
@@ -161,46 +192,63 @@ show_merge_times(char *package, const char *logfile, int average, char human_rea
 							/* a parallel emerge has finished */
 							parallel_emerge--;
 							continue;
-						} else
-							/* the main emerge was stopped */
+						} else {
+							/* the main emerge was stopped? if there's
+							 * more lines this file is just corrupt or
+							 * truncated at the front */
 							break;
+						}
 					}
 
-					/*
-					 * pay attention to malformed log files (when the end of an emerge process
-					 * is not indicated by the line '*** terminating'). We assume than the log is
-					 * malformed when we find a parallel emerge process which is trying to
-					 * emerge the same package
+					/* pay attention to malformed log files (when the
+					 * end of an emerge process is not indicated by the
+					 * line '*** terminating'). We assume that the log
+					 * is malformed when we find a parallel emerge
+					 * process which is trying to emerge the same
+					 * package
 					 */
-					if (strncmp(buf[1], ">>> emerge (", 12) == 0 && parallel_emerge > 0) {
+					if (strncmp(buf[1], ">>> emerge (", 12) == 0 &&
+							parallel_emerge > 0)
+					{
+						/* find package name */
 						p = strchr(buf[1], ')');
 						q = strchr(ep, ')');
 						if (!p || !q)
 							continue;
 
-						if (!strcmp(p, q)) {
+						/* is this emerge doing the same thing as we're
+						 * looking for? that means we failed */
+						if (strcmp(p, q) == 0) {
 							parallel_emerge--;
 							/* update the main emerge reference data */
 							snprintf(ep, BUFSIZ, "completed %s", &buf[1][4]);
+							t[0] = t[1];
 							continue;
 						}
 					}
 
-					if (strncmp(&buf[1][4], ep, BUFSIZ) == 0) {
+					/* if this line matches "completed emerge (X of Y) ..."
+					 * we're finally somewhere */
+					if (strncmp(&buf[1][4], ep, BUFSIZ - 4) == 0) {
 						if (!average) {
-							strcpy(buf[1], "");
+							buf[1][0] = '\0';
 							if (verbose) {
 								if (atom->PR_int)
-									snprintf(buf[1], sizeof(buf[1]), "-%s-r%i", atom->PV,  atom->PR_int);
+									snprintf(buf[1], sizeof(buf[1]),
+											"-%s-r%i", atom->PV,  atom->PR_int);
 								else
-									snprintf(buf[1], sizeof(buf[1]), "-%s", atom->PV);
+									snprintf(buf[1], sizeof(buf[1]),
+											"-%s", atom->PV);
 							}
-							printf("%s%s%s%s: %s: ", BLUE, atom->PN, buf[1], NORM, chop_ctime(t[0]));
+							printf("%s%s%s%s: %s: ",
+									BLUE, atom->PN, buf[1], NORM,
+									chop_ctime(t[0]));
 							if (human_readable)
 								print_seconds_for_earthlings(t[1] - t[0]);
 							else
-								printf("%s%"PRIu64"%s seconds", GREEN, (uint64_t)(t[1] - t[0]), NORM);
-							puts("");
+								printf("%s%"PRIu64"%s seconds",
+										GREEN, (uint64_t)(t[1] - t[0]), NORM);
+							printf("\n");
 						}
 						merge_time += (t[1] - t[0]);
 						count++;
@@ -232,7 +280,8 @@ show_emerge_history(int listflag, array_t *atoms, const char *logfile,
                     time_t start_time, time_t end_time)
 {
 	FILE *fp;
-	size_t buflen, linelen;
+	int linelen;
+	size_t buflen;
 	char *buf, merged;
 	char *p, *q;
 	bool showit;
@@ -246,16 +295,17 @@ show_emerge_history(int listflag, array_t *atoms, const char *logfile,
 	}
 
 	buf = NULL;
-	while ((linelen = getline(&buf, &buflen, fp)) != -1) {
+	while ((linelen = getline(&buf, &buflen, fp)) >= 0) {
 		if (linelen < 30)
 			continue;
 
-		rmspace_len(buf, linelen);
+		rmspace_len(buf, (size_t)linelen);
 		if ((p = strchr(buf, ':')) == NULL)
 			continue;
 		*p = 0;
 		q = p + 3;
-		/* Make sure there's leading white space and not a truncated string. #573106 */
+		/* Make sure there's leading white space and not a truncated
+		 * string. #573106 */
 		if (p[1] != ' ' || p[2] != ' ')
 			continue;
 
@@ -331,7 +381,8 @@ static void
 show_sync_history(const char *logfile, time_t start_time, time_t end_time)
 {
 	FILE *fp;
-	size_t buflen, linelen;
+	int linelen;
+	size_t buflen;
 	char *buf, *p;
 	time_t t;
 
@@ -342,7 +393,7 @@ show_sync_history(const char *logfile, time_t start_time, time_t end_time)
 
 	buf = NULL;
 	/* Just find the finish lines. */
-	while ((linelen = getline(&buf, &buflen, fp)) != -1) {
+	while ((linelen = getline(&buf, &buflen, fp)) >= 0) {
 		/* This cuts out like ~10% of the log. */
 		if (linelen < 35)
 			continue;
@@ -356,7 +407,7 @@ show_sync_history(const char *logfile, time_t start_time, time_t end_time)
 			continue;
 		p += 19;
 
-		rmspace_len(buf, linelen);
+		rmspace_len(buf, (size_t)linelen);
 
 		t = (time_t)atol(buf);
 		if (t < start_time || t > end_time)
@@ -379,6 +430,8 @@ show_sync_history(const char *logfile, time_t start_time, time_t end_time)
 static void show_current_emerge(void);
 #ifdef __linux__
 # include <asm/param.h>
+#endif
+#if defined __linux__ || defined __GNU__
 # include <elf.h>
 static unsigned long hz = 0;
 static void init_hz(void)
@@ -445,7 +498,9 @@ void show_current_emerge(void)
 		if (!eat_file(path, &cmdline, &cmdline_len))
 			continue;
 
-		if (cmdline[0] == '[' && (p = strchr(cmdline, ']')) != NULL && strstr(cmdline, "sandbox") != NULL) {
+		if (cmdline[0] == '[' && (p = strchr(cmdline, ']')) != NULL &&
+				strstr(cmdline, "sandbox") != NULL)
+		{
 			*p = '\0';
 			p = cmdline + 1;
 			q = p + strlen(p) + 1;
@@ -508,7 +563,9 @@ void show_current_emerge(void)
 	char *p, *q;
 	time_t start_date = 0;
 
-	if (! (kd = kvm_open("/dev/null", "/dev/null", "/dev/null", O_RDONLY, "kvm_open"))) {
+	if (! (kd = kvm_open("/dev/null", "/dev/null", "/dev/null",
+					O_RDONLY, "kvm_open")))
+	{
 		warnp("Could not open kvm: %s", kvm_geterr(kd));
 		return;
 	}
@@ -577,7 +634,8 @@ void show_current_emerge(void)
 			raip = realloc(ip, sizeof(struct kinfo_proc) * size);
 			if (raip == NULL) {
 				free(ip);
-				warnp("Could not extend allocated block to %d bytes for process information",
+				warnp("Could not extend allocated block to "
+						"%zd bytes for process information",
 						sizeof(struct kinfo_proc) * size);
 				return;
 			}
@@ -749,6 +807,7 @@ parse_date(const char *sdate, time_t *t)
 int qlop_main(int argc, char **argv)
 {
 	size_t i;
+	int ret;
 	int average = 1;
 	time_t start_time, end_time;
 	char do_time, do_list, do_unlist, do_sync, do_current, do_human_readable = 0;
@@ -761,8 +820,8 @@ int qlop_main(int argc, char **argv)
 	end_time = LONG_MAX;
 	do_time = do_list = do_unlist = do_sync = do_current = 0;
 
-	while ((i = GETOPT_LONG(QLOP, qlop, "")) != -1) {
-		switch (i) {
+	while ((ret = GETOPT_LONG(QLOP, qlop, "")) != -1) {
+		switch (ret) {
 			COMMON_GETOPTS_CASES(qlop)
 
 			case 't': do_time = 1; break;
@@ -795,7 +854,7 @@ int qlop_main(int argc, char **argv)
 
 	argc -= optind;
 	argv += optind;
-	for (i = 0; i < argc; ++i) {
+	for (i = 0; i < (size_t)argc; ++i) {
 		atom = atom_explode(argv[i]);
 		if (!atom)
 			warn("invalid atom: %s", argv[i]);
@@ -817,7 +876,7 @@ int qlop_main(int argc, char **argv)
 		show_sync_history(logfile, start_time, end_time);
 
 	if (do_time) {
-		for (i = 0; i < argc; ++i)
+		for (i = 0; i < (size_t)argc; ++i)
 			show_merge_times(argv[i], logfile, average, do_human_readable,
 				start_time, end_time);
 	}
